/* MODKEEN.C - source for a tile importer/exporter for Keen 1 - 6.
** Greetings and thanks from Andrew Durdin to Anders Gavare and 
** Daniel Olson for their assistance.
**
** Copyright (c)2007 by Ignacio R. Morelle "Shadow Master". (shadowm2006@gmail.com)
** Original code Copyright (c)2002-2004 Andrew Durdin. (andy@durdin.net)
**
** This software is provided 'as-is', without any express or implied warranty.
** In no event will the authors be held liable for any damages arising from
** the use of this software.
** Permission is granted to anyone to use this software for any purpose, including
** commercial applications, and to alter it and redistribute it freely, subject
** to the following restrictions:
**    1. The origin of this software must not be misrepresented; you must not
**       claim that you wrote the original software. If you use this software in
**       a product, an acknowledgment in the product documentation would be
**       appreciated but is not required.
**    2. Altered source versions must be plainly marked as such, and must not be
**       misrepresented as being the original software.
**    3. This notice may not be removed or altered from any source distribution.
*/

#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>

#include "utils.h"
#include "switches.h"
#include "keen123.h"
#include "keen456.h"

#include "pconio.h"

static char* txt_signature1 = 
	"LMODKEEN 2 release 1 - Windows/Linux ModKeen port, Copyright (c) 2007 by Shadow Master\n";
static char* txt_signature2 =
	"Based on ModKeen 2.0.1 source code, Copyright (c) 2002-2004 Andrew Durdin\n"
	"Based on Fin2BMP source code, Copyright (c) 2002 Andrew Durdin\n"
	"\n";

// Local function prototypes
char checkhelpparam(int argc, char* argv[]);

int main(int argc, char *argv[]) {
	SwitchStruct *switches;
	
	/* Display the signature */
	//fprintf(stdout, txt_signature1);
	//fprintf(stdout, txt_signature2);
	if (checkhelpparam(argc, argv)) return 0;
	/* Parse first, ask questions later :) */
	/* Get the options */
	switches = getswitches(argc, argv);
	
	/* Setup terminal now */
	if (pconio_init()) {
		fprintf(stderr, "CONIO: failed to setup terminal.");
		return -1;
	}
	/* TODO: instead of displaying da stupid signature again,
	 * why not write a fully-featured ncurses-based GUI? */
	/* Display the signature (again)*/
	bold;
	do_output(txt_signature1);
	unbold;
	do_output(txt_signature2);

	if(switches->Export)
	{
		/* Export all data */
		if(switches->Episode >= 1 && switches->Episode <= 3)
		{
			k123_export_begin(switches);
			k123_export_bitmaps();
			k123_export_sprites();
			k123_export_tiles();
			k123_export_fonts();
			k123_export_external();
			k123_export_end();
		}
		else if(switches->Episode >= 4 && switches->Episode <= 7)
		{
			k456_export_begin(switches);
			k456_export_fonts();
			k456_export_bitmaps();
			k456_export_masked_bitmaps();
			k456_export_tiles();
			k456_export_masked_tiles();
			k456_export_8_tiles();
			k456_export_8_masked_tiles();
			k456_export_sprites();
			k456_export_texts();
			k456_export_misc();
			k456_export_demos();
			k456_export_end();
		}
	}
 	else if(switches->Import)
	{
		/* Export all data */
		if(switches->Episode >= 1 && switches->Episode <= 3)
		{
			k123_import_begin(switches);
			k123_import_bitmaps();
			k123_import_sprites();
			k123_import_tiles();
			k123_import_fonts();
			k123_import_external();
			k123_import_end();
		}
		else if(switches->Episode >= 4 && switches->Episode <= 7)
		{
			k456_import_begin(switches);
			k456_import_fonts();
			k456_import_bitmaps();
			k456_import_masked_bitmaps();
			k456_import_tiles();
			k456_import_masked_tiles();
			k456_import_8_tiles();
			k456_import_8_masked_tiles();
			k456_import_sprites();
			k456_import_texts();
			k456_import_misc();
			k456_import_demos();
			k456_import_end();
		}
	}
	
	do_output("Done!\n\n");
	
	/* Quit, indicating success */
	return 0;
}

// Andy's getswitch function is 1000 times superior than this thing I wrote
// in 5 minutes, curses!
char checkhelpparam(int argc, char* argv[])
{
	int i = 0;
	char haveit = 0;
	if (argc) {
		while ((!haveit) && (i < argc))
		{
			if ((strcasecmp(argv[i], "-h") == 0)||
			(strcasecmp(argv[i], "-help") == 0)||
			(strcasecmp(argv[i], "--help") == 0))
				haveit = 1;
			i++;
		}
		if (!haveit) return 0;
	}
	fprintf(stdout,
			"  Valid options for LMODKEEN are:\n"
			"    -nowait             [Doesn't ask for a keypress at exit; useful for scripts]\n"
			"    -episode=N          [Episode identifier; n = 1, 2, 3, 4, 5 or 6]\n"
			"    -export             [Export data from Keen to BMP files]\n"
			"    -import             [Import data from BMP files to Keen]\n"
			"    -keendir=DIRECTORY  [Keen files are in DIRECTORY (defaults to current)]\n"
			"    -bmpdir=DIRECTORY   [BMP files will be stored in DIRECTORY (defaults to current)]\n"
			"    -backup             [Create backups of changed files]\n"
			"    -debug              [Show debug information for developers and testers]\n"
			"    -help               [Shows the valid options for MODKEEN]\n"
			"\n"
			"  EXAMPLES:\n"
#ifdef WIN32
			"    %s -episode=4 -export -bmpdir=\"C:\\My Mods\\Keen4\"\n"
#else
			"    %s -episode=4 -export -bmpdir=\"/home/joe/my_mods/keen4\"\n"
#endif /* WIN32 */
			"    %s -import -episode=1 -backup -nowait\n"
			"    %s -help\n"
			"\n"
			"Note that you must specify the -import or -export parameters if you want ModKeen to\n"
			"actually do something (these parameters cannot be used at the same time!).\n"
			"For more information on ModKeen/LModKeen, read the README file provided with the\n"
			"program's source code or binaries.\n"
			"\n", argv[0], argv[0], argv[0]);
	return 1;
}
